#ifndef CPU_Ticks_h
#define CPU_Ticks_h

#include <McadIncl.h>

//+***********************************************************************+//
// Global variable to hold the cpu tick count
//+***********************************************************************+//
extern __int64		StartTickCount;
extern double		TicksToSec;


//+***********************************************************************+//
// Function to read the time stamp counter.  The function type is cdecl by
// default, but just make it explict.  I turned off the pragma warning
// that the function does not return a value because it actually does.
//+***********************************************************************+//
#pragma warning (push)
#pragma warning (disable : 4035)	// disable no return value warning

__forceinline __int64  __cdecl ReadTimeStampCounter(void)
{
	_asm
	{
		rdtsc				// return with the 64 bit value in edx:eax
	}

}

#pragma warning (pop)


//+***********************************************************************+//
// Function to reset the global tick count variable and measure the clock
// speed.
//+***********************************************************************+//
__forceinline void InitTimeStampCounter(void)
{
	// Initialize the global tick count
	StartTickCount	= ReadTimeStampCounter();
}

//+***********************************************************************+//
// Compute the elapsed time in ticks using double precision
//+***********************************************************************+//
__forceinline double DblElapsedTicks(void)
{
	// Compute the number of elapsed ticks
	__int64	TimeDiff	= ReadTimeStampCounter() - StartTickCount;
	
	// Return the tick count in double precision
	return (double)TimeDiff;
}

//+***********************************************************************+//
// Function to calibrate the cpu clock in sec
//+***********************************************************************+//
void CalibrateTimeStampCounter(void);

//+***********************************************************************+//
// Mathcad interface functions
// Mathcad apparently doesn't like functions with no user arguments, so
// add a dummy argument.  This argument will be used in the Mathcad document
// to cause things to recalculate.
//+***********************************************************************+//

//+***********************************************************************+//
// Read the number of ticks since the global counter variable was reset
//+***********************************************************************+//
extern FUNCTIONINFO	ReadCpuTicks;

LRESULT  fReadCpuTicks(
        LPCOMPLEXSCALAR		CurrentTickCount,
		LPCCOMPLEXSCALAR	Dummy);

//+***********************************************************************+//
// Grab the current time stamp count and put it in a global variable. ie,
// start the stopwatch.
//+***********************************************************************+//
extern FUNCTIONINFO	ResetCpuTicks;

LRESULT  fResetCpuTicks(
        LPCOMPLEXSCALAR		Zero,
		LPCCOMPLEXSCALAR	Dummy);

//+***********************************************************************+//
// Compute the time in sec since the global counter variable was reset
//+***********************************************************************+//
extern FUNCTIONINFO	ReadTime;

LRESULT  fReadTime(
        LPCOMPLEXSCALAR		TimeDiffInSec,
		LPCCOMPLEXSCALAR	Dummy);


//+***********************************************************************+//
// Compute the CPU Speed in Mhz
//+***********************************************************************+//
extern FUNCTIONINFO	ReadCpuSpeed;

LRESULT  fReadCpuSpeed(
        LPCOMPLEXSCALAR		CpuMhz,
		LPCCOMPLEXSCALAR	Dummy);

//+***********************************************************************+//
// Convert clock ticks to time in sec
//+***********************************************************************+//
extern FUNCTIONINFO	CpuTicksToSec;

LRESULT  fCpuTicksToSec(
        LPCOMPLEXSCALAR		TimeInSec,
		LPCCOMPLEXSCALAR	TimeInTicks);

//+***********************************************************************+//
// Convert clock ticks to time in sec
//+***********************************************************************+//
extern FUNCTIONINFO	SecToCpuTicks;

LRESULT  fSecToCpuTicks(
        LPCOMPLEXSCALAR		TimeInTicks,
		LPCCOMPLEXSCALAR	TimeInSec);

#endif